//===========================================================================
// Texturbeispiel
//===========================================================================

//===========================================================================
// includes
//===========================================================================

// GL und GLUT Bibliotheken einbinden (je nach Betriebssystem)

#ifdef WIN32
    #include <windows.h>
    #include <GL/gl.h>
    #include <GL/glut.h>
#else
    #include <GLUT/glut.h>
#endif

// Das hier ist notwendig, um mit den GL Extensions arbeiten zu knnen

#define GLH_EXT_SINGLE_FILE
#include <glh/glh_extensions.h>

#include "texture.h"
#include "shader.h"

#include <math.h>


//===========================================================================
// Konstanten
//===========================================================================

const float        PI             = asin(1.0f)*2.0f;


//===========================================================================
// Globale variablen
//===========================================================================

int      glutWindow     = 0;
float    objectRotation = 0.0f;
CShader  shader;
CTexture original;


//===========================================================================
// OpenGL Zeichenfunktion
//===========================================================================

void transform(unsigned int width, unsigned int height)
{
    // OpenGL Transformationen initialisieren...
    // Ausgabefenstergre setzen
    glViewport(0, 0, (GLint)width, (GLint)height);

    // Verhltnis Breite zu Hhe (aspect) berechnen
    GLdouble aspect = (GLdouble)width / (GLdouble)height;
    
    // Perspektivische Projektion bearbeiten
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    // Kameraperspektive setzen
    GLdouble fieldOfView = atan(1.0/2.0)*180.0/double(PI)*2.0;
    gluPerspective(fieldOfView,  // ffnungswinkel vertikal (field of view)
                   aspect,       // aspect ratio setzen
                   0.001, 10.0); // nahe und ferne Begrenzungsebene setzen

    // Transformationen fr 3D-Koordinaten bearbeiten
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    // Kameraposition setzen
    gluLookAt(0.0, 0.0, 2.0,  // von wo
              0.0, 0.0, 0.0,  // nach wo
              0.0, 1.0, 0.0); // wo ist oben?
}


void update()
{
    float deltaTime = 1.0f/60.0f;

    // rotiere ein Mal in 40 Sekunden
    objectRotation  += deltaTime*360.0f/40.0f;
}


void render()
{
    original.use();

    shader.use();
    shader.setUniform("texture", 0);

    glPushMatrix();
    glRotatef(objectRotation, 2.0f, 1.0f, 0.0f);
    glutSolidTeapot(0.5f);
    glPopMatrix();

    shader.disable();

    original.disable();
}


//===========================================================================
// GLUT callback funktionen
//===========================================================================

void display(void)
{
    // Ausgabefenster in schwarz lschen
    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    // Transformationen vor dem Zeichnen setzen
    transform(glutGet(GLUT_WINDOW_WIDTH), glutGet(GLUT_WINDOW_HEIGHT));

    // Simulation aktualisieren
    update();

    // Zeichnen
    render();

    // Hintergrundpuffer anzeigen
    glutSwapBuffers();
}


void idle(void)
{
    // nochmaliges Zeichnen einleiten
    if (glutWindow != 0) glutPostRedisplay();
}


void keyboard(unsigned char key, int x, int y)
{
    // Fenster schlieen
    glutDestroyWindow(glutWindow);
    glutWindow = 0;

    // Programm beenden (etwas unsauber)
    exit(0);
}


//===========================================================================
// Hauptroutine
//===========================================================================

int main(int argc, char* argv[])
{
    // GLUT initialisieren
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);

    // Fenster ffnen
    glutWindow = glutCreateWindow("shader");

    // GLUT Funktionen fr Zeichnen, Leerlauf, Tastatur und Maus setzen
    glutDisplayFunc(display);
    glutIdleFunc(idle);
    glutKeyboardFunc(keyboard);

    // Vollbildmodus ohne Cursor
    glutFullScreen();
    glutSetCursor(GLUT_CURSOR_NONE);

    // Rest initialisieren
    glEnable(GL_DEPTH_TEST);

    // Textur laden
    original.load("../../Elevation.bmp");
    original.upload();

    // Shader laden
    shader.loadShaders("../../environment.vert", "../../environment.frag");

    // Hauptschleife des Programms aufrufen
    glutMainLoop();

    // Bis hierhin kommt das Programm nicht
    return 0;
}
