#ifndef _SHADER_H_
#define _SHADER_H_

//===========================================================================
// includes
//===========================================================================

// GL Bibliothek einbinden (je nach Betriebssystem)

#ifdef WIN32
    #include <windows.h>
    #include <GL/gl.h>
#else
    #include <OpenGL/glut.h>
#endif

//=============================================================================
// class
//=============================================================================

// OpenGL Shader class
class CShader
{
  public:

    CShader();
    ~CShader();

    /**
     * @brief  Load shaders from file
     *
     * This loads the shaders from file and loads them into the
     * graphics card, compiles and links them.
     *
     * @param  vertexName   [in] Filename for vertex shader code
     * @param  fragmentName [in] Filename for fragment shader code
     * @return true on success, false on failure
     */
    bool loadShaders(const char* vertexName, const char* fragmentName);

    /**
     * @brief  Use shaders in rendering
     * @return true on success, false on failure
     */
    bool use();

    /**
     * @brief  Set uniform variable
     * @param  name  [in] Name of variable
     * @param  value [in] Value to set
     * @return true on success, false on failure
     */
    bool setUniform(const char* name, const int value);
    /**
     * @brief  Set uniform variable
     * @param  name  [in] Name of variable
     * @param  value [in] Value to set
     * @return true on success, false on failure
     */
    bool setUniform(const char* name, const float value);
    /**
     * @brief  Set 3D uniform variable
     * @param  name   [in] Name of variable
     * @param  value1 [in] first value to set
     * @param  value2 [in] second value to set
     * @return true on success, false on failure
     */
    bool setUniform(const char* name, const float value1, const float value2);
    /**
     * @brief  Set 3D uniform variable
     * @param  name   [in] Name of variable
     * @param  value1 [in] first value to set
     * @param  value2 [in] second value to set
     * @param  value3 [in] third value to set
     * @return true on success, false on failure
     */
    bool setUniform(const char* name, const float value1, const float value2, const float value3);

    /**
     * @brief  Disable shaders use in rendering
     * @return true on success, false on failure
     */
    bool disable();

  protected:

    static void printShaderInfoLog(GLuint obj);
    static void printProgramInfoLog(GLuint obj);

    GLuint m_vertexShader;
    GLuint m_fragmentShader;
    GLuint m_shaderProg;    
};

#endif // _SHADER_H_
